# Copyright (c) Microsoft Corporation.
# Licensed under the MIT license.

from pyrit.prompt_target.prompt_target import PromptTarget
from pyrit.prompt_target.azure_openai_chat_target import AzureOpenAIChatTarget


__all__ = ["AzureOpenAIChatTarget", "PromptTarget"]
